//@dart=2.9
import 'dart:async';
import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:flutter_local_notifications/flutter_local_notifications.dart';
import 'package:get_version/get_version.dart';
import 'package:pusher_websocket_flutter/pusher.dart';
import '../pay_unit_sdk.dart';
import 'blocs/PayUnitStream.dart';
import 'package:flutter/cupertino.dart';
import 'package:http/http.dart' as http;
import 'package:native_progress_hud/native_progress_hud.dart';
import './Constant/Constant.dart'as constant;


class ApiService {
  Channel channel;
  final localNotificationsPlugin = FlutterLocalNotificationsPlugin();

  Future<String> initPlatformState() async {
    String projectAppID;
    try {
      projectAppID = await GetVersion.appID;
      print("this is app id : $projectAppID");
      return projectAppID;
    } catch (e) {
      return projectAppID = 'Failed to get app ID.';
    }

  }


  initializeNotifications() async {
    final initializeAndroid =
    AndroidInitializationSettings('@mipmap/ic_launcher');
    final initializeIOS = IOSInitializationSettings();
    final initSettings =
    InitializationSettings(android: initializeAndroid, iOS: initializeIOS);
    await localNotificationsPlugin.initialize(initSettings);
  }

  Future<void> singleNotification(String title, String body) async {
    final androidChannel = AndroidNotificationDetails(
      'channel-id',
      'PayUniy',
      'Scheduled notifications',
      importance: Importance.max,
      priority: Priority.max,
      color: Color(0xFF3AD9F2),
      visibility: NotificationVisibility.public,
      enableVibration: false,
      styleInformation: BigTextStyleInformation(''),
    );
    final iosChannel = IOSNotificationDetails();
    final platformChannel =
    NotificationDetails(android: androidChannel, iOS: iosChannel);
    await localNotificationsPlugin.show(3, title, body, platformChannel,
        payload: "");
  }

  Future<void> cancelNotification() async {
    return await localNotificationsPlugin.cancel(0);
  }

  Future<void> cancelAllNotifications() async {
    return await localNotificationsPlugin.cancelAll();
  }

  initiatePayment({
    @required transaction_id,
    @required String transactionAmount,
    @required String currency,
    @required String transactionCallBackUrl,
    @required context,
    @required String X_API_KEY,
    @required String productName,
    @required String description,
    @required merchandUserName,
    @required merchandPassword,
    @required mode,
    @required Function actionAfterProccess,
  }) async {
    try {
      progressHub("Please wait", context);
      await initPusher(transaction_id, actionAfterProccess, context);
      print("This is the application_id on apiservice : ${await initPlatformState()}");
      var data = jsonEncode({
        "currency": currency,
        "transaction_id": transaction_id,
        "total_amount": transactionAmount,
        "return_url": await initPlatformState()
      });
      var jsonResponse;
      var url = Uri.parse("${constant.baseUrl}/gateway/initialize");
      var response = await http.post(url,
          body: data,
          headers: getGlobalHeader(
              X_API_KEY, "$merchandUserName:$merchandPassword", mode));
      jsonResponse = json.decode(response.body);
      if (response.statusCode == 200 || response.statusCode == 201) {
        if (jsonResponse != null) {
          List result = await getAllProviders(
            context: context,
            transactionId: jsonResponse['data']['t_id'],
            transactionAmount: jsonResponse['data']['t_sum'],
            transactionCallBackUrl: jsonResponse['data']['t_url'],
            merchandPassword: merchandPassword,
            merchandUserName: merchandUserName,
            X_API_KEY: X_API_KEY,
            mode: mode,
            actionAfterProccess: actionAfterProccess,
          );
          return {"result": result, "transaction_id": transaction_id};
        }
      } else {
        print("Response : $jsonResponse");
        if (jsonResponse['message'] == null) {
          constant.makeToast(jsonResponse['error'], context, Colors.red);
        } else {
          constant.makeToast(jsonResponse['message'], context, Colors.red);
        }
        actionAfterProccess(transaction_id, "FAILED");
        return false;
      }
    } catch (e) {
      print("Error : $e");
      constant.makeToast("Something went wrong , please try again or later",
          context, Colors.red);
      actionAfterProccess(transaction_id, "FAILED");
      return false;
    } finally {
      closeDialog(context);
    }
  }

  getAllProviders({
    context,
    @required String transactionId,
    @required String transactionAmount,
    @required String transactionCallBackUrl,
    @required String merchandUserName,
    @required String merchandPassword,
    @required String X_API_KEY,
    @required String mode,
    @required Function actionAfterProccess,
  }) async {
    try {
      var jsonResponse;
      var url = Uri.parse(
          "${constant.baseUrl}/gateway/gateways?t_id=$transactionId&t_sum=$transactionAmount&t_url=$transactionCallBackUrl");
      var response = await http.get(url,
          headers: getGlobalHeader(
              X_API_KEY, "$merchandUserName:$merchandPassword", mode));
      jsonResponse = json.decode(response.body);
      if (response.statusCode == 200) {
        if (jsonResponse != null) {
          return json.decode(response.body)['data'];
        }
      } else {
        if (jsonResponse != null) {
          print("Response :  : $jsonResponse");
          closeDialog(context);
          actionAfterProccess(transactionId, "FAILED");
          return [];
        }
      }
    } catch (e) {
      print("Error : $e");
      closeDialog(context);
      constant.makeToast("Something went wrong , please try again or later",
          context, Colors.red);
      actionAfterProccess(transactionId, "FAILED");
    } finally {
      closeDialog(context);
    }
  }

  makePayment({
    @required context,
    @required String X_API_KEY,
    @required String transactionAmount,
    @required String transactionCallBackUrl,
    @required String provider_short_tag,
    @required String phoneNumber,
    @required Function actionAfterProccess,
    @required String userName,
    @required merchandUserName,
    @required transaction_id,
    @required merchandPassword,
    @required mode,
    @required currency,
    @required paymentType,
  }) async {
    var response;
    var data = currency != null
        ? jsonEncode({
      'gateway': provider_short_tag,
      'amount': int.parse(transactionAmount),
      'transaction_id': transaction_id,
      'return_url': transactionCallBackUrl,
      'phone_number': phoneNumber,
      "name": userName,
      "currency": currency,
      "paymentType": paymentType
    })
        : jsonEncode({
      'gateway': provider_short_tag,
      'amount': int.parse(transactionAmount),
      'transaction_id': transaction_id,
      'return_url': transactionCallBackUrl,
      'phone_number': phoneNumber,
      "name": userName,
      "paymentType": paymentType
    });
    try {
      progressHub("Please wait", context);
      var jsonResponse;
      if (provider_short_tag != "paypal") {
        constant.closeAndToastPaymentAreMake(context);
      }
      var url = Uri.parse("${constant.baseUrl}/gateway/makepayment");
      response = await http.post(url,
          body: data,
          headers: getGlobalHeader(
              X_API_KEY, "$merchandUserName:$merchandPassword", mode));
      if (response.statusCode == 200 || response.statusCode == 201) {
        constant.makeToast(
            json.decode(response.body)['message'], context, Colors.green);
        jsonResponse = json.decode(response.body)["data"];
        if (jsonResponse["access_token"] != null) {
          var obj = {
            "order_id": jsonResponse["order_id"],
            "amount": jsonResponse["amount"],
            "currency": jsonResponse["currency"],
            "client_id": jsonResponse["client_id"],
            "accessToken": jsonResponse["access_token"],
          };
          getPaymentStatus(
              transactionId: transaction_id,
              paymentRef: jsonResponse["payment_ref"],
              payToken: jsonResponse["pay_token"],
              merchandUserName: merchandUserName,
              merchandPassword: merchandPassword,
              provider_short_tag: provider_short_tag,
              actionAfterProccess: actionAfterProccess,
              context: context,
              mode: mode,
              X_API_KEY: X_API_KEY);
          return obj;
        } else {
          var obj = {
            "order_id": jsonResponse["order_id"],
            "amount": jsonResponse["amount"],
            "currency": jsonResponse["currency"],
            "client_id": jsonResponse["client_id"],
          };
          getPaymentStatus(
              transactionId: transaction_id,
              paymentRef: jsonResponse["payment_ref"],
              payToken: jsonResponse["pay_token"],
              merchandUserName: merchandUserName,
              merchandPassword: merchandPassword,
              provider_short_tag: provider_short_tag,
              actionAfterProccess: actionAfterProccess,
              context: context,
              mode: mode,
              X_API_KEY: X_API_KEY);
          return obj;
        }
      } else {
        var responseData = json.decode(response.body);
        print(" Response : $responseData");
        actionAfterProccess(transaction_id, "FAILED");
        if (responseData != null) {
          payUnitStream.paymentSink.addError("Error");
          closeDialog(context);
          constant.makeToast(responseData['message'], context, Colors.red);
        }
      }
    } catch (e) {
      actionAfterProccess(transaction_id, "FAILED");
      print("Error : ${response.body}");

      closeDialog(context);
      constant.makeToast("Something went wrong , please try again or later",
          context, Colors.red);
    } finally {}
  }

  getPaymentStatus({
    @required context,
    @required String transactionId,
    @required String paymentRef,
    @required String payToken,
    @required String merchandUserName,
    @required String merchandPassword,
    @required String provider_short_tag,
    @required Function actionAfterProccess,
    @required String X_API_KEY,
    @required String mode,
  }) async {
    try {
      const time = const Duration(seconds: 4);
      Timer _timer;
      _timer = new Timer.periodic(time, (Timer timer) async {
        var jsonResponse;
        var url = Uri.parse(
            "${constant.baseUrl}/gateway/paymentstatus/$provider_short_tag/$transactionId?payment_ref=${paymentRef}&pay_token=${payToken}");
        var response = await http.get(url,
            headers: getGlobalHeader(
                X_API_KEY, "$merchandUserName:$merchandPassword", mode));
        if (response.statusCode == 200) {
          _timer.cancel();
        } else {
          if (jsonResponse != null) {
            print("Response : $jsonResponse");
            constant.makeToast(jsonResponse['error'], context, Colors.red);

            closeDialog(context);
            _timer.cancel();
          }
        }
      });
    } catch (e) {
      constant.makeToast("Something went wrong , please try again or later",
          context, Colors.red);
    } finally {}
  }

  Future<void> initPusher(
      channelName, actionAfterProccess, BuildContext context) async {
    try {
      await Pusher.init('8e6ec079f9d817bd73eb', PusherOptions(cluster: 'ap4'));
    } catch (e) {}
    Pusher.connect(
        onConnectionStateChange:
            (ConnectionStateChange connectionState) async {},
        onError: (ConnectionError e) {});

    channel = await Pusher.subscribe(channelName);
    channel.bind('payment-event', (last) {
      if (jsonDecode(last.data)['transaction_status'] == "SUCCESS") {
        singleNotification('PayUnit',
            'Transaction :${jsonDecode(last.data)['transaction_id']} , is success');
        payUnitStream.paymentSink.add(false);

        closeDialog(context);
        constant.makeToast(
            "Your transaction : ${jsonDecode(last.data)['transaction_id']} have the status: ${jsonDecode(last.data)['transaction_status']}",
            context,
            jsonDecode(last.data)['transaction_status'] == "SUCCESS"
                ? Colors.green
                : Colors.red);
        actionAfterProccess(jsonDecode(last.data)['transaction_id'],
            jsonDecode(last.data)['transaction_status']);
        closeDialog(context);
      } else {
        closeDialog(context);
        singleNotification('PayUnit',
            'Transaction No :${jsonDecode(last.data)['transaction_id']} , is failed');
        constant.makeToast(
            "Your transaction failed , please try again or later",
            context,
            Colors.red);
        payUnitStream.paymentSink.addError("error");
      }
    });
  }

  void unbindEvent() {
    channel.unbind('payment-event');
  }

  void unSubscribePusher(String channelName) {
    Pusher.unsubscribe(channelName);
  }

  String getEncode(String apiUserNameAndPassWord) {
    String credentials = "${apiUserNameAndPassWord}";
    Codec<String, String> stringToBase64 = utf8.fuse(base64);
    String encoded = stringToBase64.encode(credentials);
    return encoded;
  }

  getGlobalHeader(
      String X_API_KEY, String apiUserNameAndPassWord, String mode) {
    return {
      'Authorization': 'Basic ${getEncode(apiUserNameAndPassWord)}',
      'Content-Type': 'application/json',
      'X-API-KEY': X_API_KEY,
      'mode': mode,
    };
  }

  progressHub(msg, context) {
    NativeProgressHud.showWaitingWithText(msg);
  }

  closeDialog(context) {
    NativeProgressHud.hideWaiting();
  }

// Future<String> createCheckout(
//     {@required amount,
//     @required productName,
//     @required qty,
//     @required currency,
//     @required secretKey,
//     @required context,
//     @required transaction_id}) async {
//   final auth = 'Basic ' + base64Encode(utf8.encode('$secretKey:'));
//   //print(" transaction_id in checkout: $transaction_id");
//   final body = {
//     'payment_method_types': ['card'],
//     'line_items': [
//       {
//         'quantity': qty,
//         'amount': amount,
//         'currency': currency,
//         'name': productName,
//       }
//     ],
//     'mode': 'payment',
//     'success_url': 'https://success.com',
//     'cancel_url': 'https://cancel.com',
//   };
//
//   try {
//     final result = await Dio().post(
//       "https://api.stripe.com/v1/checkout/sessions",
//       data: body,
//       options: Options(
//         headers: {HttpHeaders.authorizationHeader: auth},
//         contentType: "application/x-www-form-urlencoded",
//       ),
//     );
//     return result.data['id'];
//   } on DioError catch (e, s) {
//     //print("SessionId Error : ${e.response}");
//     closeDialog(context);
//     constant.makeToast("Something went wrong , please try again or later",
//         context, Colors.red);
//
//     throw e;
//   }
// }
//
// Future<String> stripeCallBackTransactionIdAndStatus(
//     {@required transactionId,
//     @required transactionStatus,
//     @required context,
//     @required merchandUserName,
//     @required merchandPassword,
//     @required sandbox,
//     @required X_API_KEY}) async {
//   var data = jsonEncode({
//     "transaction_id": transactionId,
//     "transaction_status": transactionStatus
//   });
//   //print("This is my data : $data");
//   progressHub("Please wait", context);
//   var response = await getDigestAuth(merchandUserName, merchandPassword).post(
//       "${sandbox == 'sandbox' ? constant.baseUrlSandBox : constant.baseUrl}/gateway/stripeCallbackMobile",
//       body: data,
//       headers: getGlobalHeader(X_API_KEY));
//   var jsonResponse = json.decode(response.body);
//   //print("This is stripeCallbackMobile global response  : ${response.body}");
//   //print("This is the status code : ${response.statusCode}");
//   try {
//     if (response.statusCode == 200 || response.statusCode == 201) {
//       //print(
//           "200 response /stripeCallBackTransactionIdAndStatus : $jsonResponse");
//       if (transactionStatus == "SUCCESS") {
//         makeToast("Paypal payment success", context, Colors.green);
//       } else {
//         makeToast("Paypal payment Failed", context, Colors.red);
//       }
//     } else {
//       //print("Else response /stripeCallBackTransactionIdAndStatus : $jsonResponse");
//       closeDialog(context);
//     }
//   } catch (e) {
//     //print("StripeCallBack error => $e");
//     constant.makeToast( "Something went wrong , please try again or later" , context , Colors.red);
//     closeDialog(context);
//   } finally {
//     closeDialog(context);
//   }
// }
}
