//@dart=2.9
import 'package:awesome_dialog/awesome_dialog.dart';
import 'package:modal_bottom_sheet/modal_bottom_sheet.dart';
import 'package:pay_unit_sdk/src/CheckoutPage.dart';
import 'package:pay_unit_sdk/src/CheckoutPaypalPage.dart';
import '../pay_unit_sdk.dart';
import 'ApiService.dart';
import './blocs/PayUnitStream.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:native_progress_hud/native_progress_hud.dart';
import 'package:simple_animations/simple_animations.dart';
import 'package:flutter/services.dart';
import 'package:intl/intl.dart';
import './Constant/Constant.dart' as constant;


class PayUnitButton extends StatefulWidget {
  final Function(String transactionId, String transactionStatus)
  actionAfterProccess;
  final String productName;
  final IconData icon;
  final double width;
  final bool isFixedHeight;
  final String transactionId;
  final Color color;
  final Color buttonTextColor;
  final String transactionAmount;
  final String transactionCallBackUrl;
  final String apiUser;
  final String apiPassword;
  final String apiKey;
  final String mode;
  final String notiFyUrl;
  final String currency;

  const PayUnitButton({
    @required this.actionAfterProccess,
    @required this.productName,
    @required this.color,
    @required this.transactionId,
    @required this.buttonTextColor,
    @required this.transactionAmount,
    @required this.transactionCallBackUrl,
    @required this.apiUser,
    @required this.apiPassword,
    @required this.apiKey,
    @required this.mode,
    @required this.notiFyUrl,
    @required this.currency,
    this.width = double.infinity,
    this.isFixedHeight = true,
    this.icon,
  });

  @override
  _PayUnitButtonState createState() => _PayUnitButtonState();
}

class _PayUnitButtonState extends State<PayUnitButton> with AnimationMixin {
  Animation<double> _scale;
  ApiService apiService = ApiService();

  @override
  void initState() {
    super.initState();
    apiService.initializeNotifications();
    final curveAnimation = CurvedAnimation(
        parent: controller, curve: Curves.easeIn, reverseCurve: Curves.easeIn);
    _scale = Tween<double>(begin: 1, end: 0.9).animate(curveAnimation);
  }

  void _onTapDown(TapDownDetails details) {
    controller.play(duration: Duration(milliseconds: 150));
  }

  void _onTapUp(TapUpDetails details) {
    if (controller.isAnimating) {
      controller.addStatusListener((status) {
        if (status == AnimationStatus.completed)
          controller.playReverse(duration: Duration(milliseconds: 100));
      });
    } else
      controller.playReverse(duration: Duration(milliseconds: 100));
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () async {
        setState(() {
          widget.transactionId;
        });
        ApiService api = new ApiService();
        var result = await api.initiatePayment(
            transactionAmount: widget.transactionAmount,
            currency: widget.currency,
            transactionCallBackUrl: widget.transactionCallBackUrl,
            X_API_KEY: widget.apiKey,
            productName: widget.productName,
            context: context,
            transaction_id: widget.transactionId,
            description: "Pay unit mobile payment",
            merchandPassword: widget.apiPassword,
            mode: widget.mode,
            merchandUserName: widget.apiUser,
            actionAfterProccess: widget.actionAfterProccess);
        if (result == false) {
          constant.makeToast("Something went wrong , please try again or later",
              context, Colors.red);
        } else {
          print("This is the result : $result");
          showMaterialModalBottomSheet(
            context: context,
            builder: (context) => PayDialog(
              X_API_KEY: widget.apiKey,
              productName: widget.productName,
              transactionCallBackUrl: widget.transactionCallBackUrl,
              transactionAmount: widget.transactionAmount,
              actionAfterProccess: widget.actionAfterProccess,
              merchandPassword: widget.apiPassword,
              merchandUserName: widget.apiUser,
              sandbox: widget.mode,
              currency: widget.currency,
              data: result,
            ),
          );
        }
      },
      onTapDown: _onTapDown,
      onTapUp: _onTapUp,
      onTapCancel: () {
        controller.playReverse(
          duration: Duration(milliseconds: 100),
        );
      },
      child: Transform.scale(
        scale: _scale.value,
        child: _animatedButtonUI,
      ),
    );
  }

  Widget get _animatedButtonUI => Container(
    height: widget.isFixedHeight ? 50.0 : null,
    width: widget.width,
    padding: const EdgeInsets.all(8.0),
    decoration: BoxDecoration(
        borderRadius: BorderRadius.all(Radius.circular(10)),
        color: widget.color ?? Theme.of(context).primaryColor),
    child: Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: <Widget>[
        Flexible(
          fit: FlexFit.loose,
          child: Text(
            'Pay with PayUnit',
            textAlign: TextAlign.center,
            style: TextStyle(
                color: widget.buttonTextColor == null
                    ? Colors.white
                    : widget.buttonTextColor,
                fontWeight: FontWeight.bold,
                fontSize: 18),
          ),
        ),
      ],
    ),
  );
}

class PayDialog extends StatefulWidget {
  final Function actionAfterProccess;
  final String X_API_KEY;
  final transactionAmount;
  final productName;
  final transactionCallBackUrl;
  final String merchandUserName;
  final String merchandPassword;
  final String sandbox;
  final String currency;
  final dynamic data;

  const PayDialog({
    @required this.X_API_KEY,
    @required this.productName,
    @required this.transactionAmount,
    @required this.transactionCallBackUrl,
    @required this.actionAfterProccess,
    @required this.merchandUserName,
    @required this.merchandPassword,
    @required this.sandbox,
    @required this.currency,
    @required this.data,
  });

  @override
  _PayDialogState createState() => _PayDialogState();
}

class _PayDialogState extends State<PayDialog> with AnimationMixin {
  ApiService api = new ApiService();
  ScrollController scrollController = new ScrollController();
  var initialValue;

  @override
  void initState() {
    super.initState();

    if(widget.data["result"].length != 0){
      initialValue = widget.data["result"] == [] ? "" : widget.data["result"].first;
    }

  }

  @override
  Widget build(BuildContext context) {
    return Container(
        height: MediaQuery.of(context).size.height / 2.2,
        // color: Colors.grey,
        child: Column(
          children: [
            widget.data["result"].length <= 0
                ? Center(
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: <Widget>[
                    SizedBox(
                      height: MediaQuery.of(context).size.height * 0.09,
                    ),
                    Container(
                        height: MediaQuery.of(context).size.height * 0.15,
                        child: Text(
                          "Transaction error , please try again later",
                          style: TextStyle(color: Colors.black,fontWeight: FontWeight.bold,fontSize: 16),
                        )),
                    SizedBox(
                      height: MediaQuery.of(context).size.height * 0.05,
                    ),
                  ],
                ))
                : Padding(
              padding: const EdgeInsets.symmetric(horizontal: 10),
              child: SingleChildScrollView(
                child: Column(
                  children: [
                    SizedBox(
                      height: MediaQuery.of(context).size.width * 0.12,
                    ),
                    Text(
                      "Choice your payment method",
                      style: TextStyle(
                          color: Colors.black,
                          fontSize: 18,
                          fontWeight: FontWeight.bold),
                    ),
                    SizedBox(
                      height: 20,
                    ),
                    Container(
                      padding: EdgeInsets.symmetric(
                          horizontal: 8, vertical: 2),
                      decoration: BoxDecoration(
                          color: Colors.white,
                          boxShadow: [
                            BoxShadow(
                              color: Colors.grey,
                              offset: Offset(0.0, 1.0), //(x,y)
                              blurRadius: 6.0,
                            ),
                          ],
                          borderRadius: BorderRadius.circular(10)),
                      // dropdown below..
                      child: DropdownButton<dynamic>(
                        isExpanded: true,
                        value: initialValue,
                        icon: Icon(Icons.arrow_drop_down,color: constant.payUnitGreenColor,),
                        iconSize: 42,
                        underline: SizedBox(),
                        items: widget.data['result']
                            .map<DropdownMenuItem<dynamic>>((value) {
                          return DropdownMenuItem<dynamic>(
                            value: value,
                            child: Row(
                              mainAxisAlignment: MainAxisAlignment.start,
                              children: [
                                Container(
                                  child: Padding(
                                    padding: const EdgeInsets.all(8.0),
                                    child: Image(
                                      image: NetworkImage(
                                        value['provider_logo'],
                                      ),
                                      fit: BoxFit.fitWidth,
                                    ),
                                  ),
                                  width: 70,
                                  height: 50,
                                ),
                                SizedBox(
                                  width: 30,
                                ),
                                Text(
                                  value['provider_name'],
                                  style: TextStyle(
                                      color: Colors.black,
                                      fontSize: 12,
                                      fontWeight: FontWeight.bold),
                                ),
                                SizedBox(),
                              ],
                            ),
                          );
                        }).toList(),
                        onChanged: (newValue) {
                          setState(() {
                            initialValue = newValue;
                            print("This is the value $initialValue");
                          });
                        },
                      ),
                    ),
                    SizedBox(
                      height: 20,
                    ),
                    GestureDetector(
                      onTap: () async {
                        String provider_short_tag =
                        initialValue['provider_short_tag'];
                        if (provider_short_tag == "mtnmomo" ||
                            provider_short_tag == "orange") {
                          Navigator.pop(context);
                          showMaterialModalBottomSheet(
                            context: context,
                            builder: (context) =>
                                PayDialogWithMomo(
                                  X_API_KEY: widget.X_API_KEY,
                                  productName: widget.productName,
                                  transactionCallBackUrl:
                                  widget.transactionCallBackUrl,
                                  transactionAmount: widget.transactionAmount,
                                  actionAfterProccess:
                                  widget.actionAfterProccess,
                                  provider_short_tag:
                                  initialValue['provider_short_tag'],
                                  merchandUserName: widget.merchandUserName,
                                  merchandPassword: widget.merchandPassword,
                                  transaction_id:
                                  widget.data["transaction_id"],
                                  sandbox: widget.sandbox,
                                ),
                          );
                        } else if (provider_short_tag == "eu") {
                          showMaterialModalBottomSheet(
                            context: context,
                            builder: (context) =>
                                PayDialogWithExpressUnion(
                                  X_API_KEY: widget.X_API_KEY,
                                  productName: widget.productName,
                                  transactionCallBackUrl:
                                  widget.transactionCallBackUrl,
                                  transactionAmount: widget.transactionAmount,
                                  actionAfterProccess:
                                  widget.actionAfterProccess,
                                  provider_short_tag:
                                  initialValue['provider_short_tag'],
                                  merchandUserName: widget.merchandUserName,
                                  merchandPassword: widget.merchandPassword,
                                  transaction_id:
                                  widget.data["transaction_id"],
                                  sandbox: widget.sandbox,
                                ),
                          );
                        } else if (provider_short_tag == 'stripe') {

                          var payUnitResult = await api.makePayment(
                              X_API_KEY: widget.X_API_KEY,
                              transactionAmount: widget.transactionAmount,
                              transactionCallBackUrl:
                              widget.transactionCallBackUrl,
                              phoneNumber: "6562090008",
                              provider_short_tag: "paypal",
                              userName: "",
                              merchandPassword: widget.merchandPassword,
                              merchandUserName: widget.merchandUserName,
                              mode: widget.sandbox,
                              transaction_id:
                              widget.data["transaction_id"],
                              context: context,
                              currency: widget.currency,
                              actionAfterProccess:
                              widget.actionAfterProccess,
                              paymentType: "card");

                          if (payUnitResult["accessToken"] == "" ||
                              payUnitResult["accessToken"] == null) {
                            constant.makeToast(
                                "Something went wrong , please try again or later",
                                context,
                                Colors.red);

                          } else {
                            Navigator.of(context).pop();
                            Navigator.of(context).push(MaterialPageRoute(
                                builder: (_) => CheckoutPage(
                                    orderId: payUnitResult["order_id"],
                                    accessToken:
                                    payUnitResult["accessToken"],
                                    actionAfterProccess:
                                    widget.actionAfterProccess,
                                    currency: widget.currency,
                                    transactionId:
                                    widget.data["transaction_id"],
                                    client_id: payUnitResult["client_id"],
                                    merchandUserName:
                                    widget.merchandUserName,
                                    merchandPassword:
                                    widget.merchandPassword,
                                    amount: widget.transactionAmount,
                                    sandbox: widget.sandbox,
                                    X_API_KEY: widget.X_API_KEY)));
                            makeToast(
                                "Please wait the loading of the page",
                                context,
                                Colors.black);
                          }
                        } else if (provider_short_tag == 'paypal') {
                          var payUnitResult = await api.makePayment(
                              paymentType: null,
                              X_API_KEY: widget.X_API_KEY,
                              currency: widget.currency,
                              transactionAmount: widget.transactionAmount,
                              transactionCallBackUrl:
                              widget.transactionCallBackUrl,
                              phoneNumber: null,
                              provider_short_tag: "paypal",
                              userName: "",
                              merchandPassword: widget.merchandPassword,
                              merchandUserName: widget.merchandUserName,
                              mode: widget.sandbox,
                              transaction_id:
                              widget.data["transaction_id"],
                              context: context,
                              actionAfterProccess:
                              widget.actionAfterProccess);
                          if (payUnitResult["order_id"] == "" ||
                              payUnitResult["order_id"] == null) {
                            constant.makeToast(
                                "Something went wrong , please try again or later",
                                context,
                                Colors.red);
                          } else {
                            makeToast(
                                "Please wait the loading of the page",
                                context,
                                Colors.black);
                            closeDialog(context);
                            Navigator.of(context).pop();
                            Navigator.of(context).push(MaterialPageRoute(
                                builder: (_) => CheckoutPaypalPage(
                                    orderId: payUnitResult["order_id"],
                                    currency: widget.currency,
                                    actionAfterProccess:
                                    widget.actionAfterProccess,
                                    transactionId:
                                    widget.data["transaction_id"],
                                    client_id: payUnitResult["client_id"],
                                    merchandUserName:
                                    widget.merchandUserName,
                                    merchandPassword:
                                    widget.merchandPassword,
                                    amount: widget.transactionAmount,
                                    sandbox: widget.sandbox,
                                    X_API_KEY: widget.X_API_KEY)));


                          }

                        } else if (provider_short_tag == "yup") {
                          makeToast(
                              "Not yet available", context, Colors.black);
                        }
                      },
                      child: Container(
                        height:
                        MediaQuery.of(context).size.height * 0.075,
                        padding: const EdgeInsets.all(8.0),
                        decoration: BoxDecoration(
                            borderRadius:
                            BorderRadius.all(Radius.circular(10)),
                            color: Color(0xFF05AB9F)),
                        child: Center(
                          child: Flexible(
                            fit: FlexFit.loose,
                            child: Text(
                              'CONTINUE',
                              textAlign: TextAlign.center,
                              style: TextStyle(
                                  color: Colors.white,
                                  fontWeight: FontWeight.w700,
                                  fontSize: 14),
                            ),
                          ),
                        ),
                      ),
                    )
                  ],
                ),
              ),
            ),
            SizedBox(
              height: MediaQuery.of(context).size.height * 0.06,
            ),
          ],
        ));
  }
}

class PayDialogWithMomo extends StatefulWidget {
  final String X_API_KEY;
  final String transactionAmount;
  final Function actionAfterProccess;
  final String provider_short_tag;
  final String phoneNumber;
  final String transaction_id;
  final String transactionCallBackUrl;
  final String merchandUserName;
  final String merchandPassword;
  final String sandbox;
  final String productName;
  final String currency;

  const PayDialogWithMomo({
    this.X_API_KEY,
    this.productName,
    this.transactionAmount,
    this.actionAfterProccess,
    this.provider_short_tag,
    this.transactionCallBackUrl,
    this.phoneNumber,
    this.transaction_id,
    this.merchandUserName,
    this.merchandPassword,
    this.sandbox,
    this.currency,
  });

  @override
  _PayDialogWithTelcoState createState() => _PayDialogWithTelcoState();
}

class _PayDialogWithTelcoState extends State<PayDialogWithMomo>
    with AnimationMixin {
  final phoneController = TextEditingController();
  final GlobalKey<FormState> _formKey = GlobalKey<FormState>();
  bool _autoValidate = false;

//////////////////Methods

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return SingleChildScrollView(
      child: Center(
          child: StreamBuilder(
              initialData: false,
              stream: payUnitStream.paymentStream,
              builder: (context, snapshot) {
                if (snapshot.hasError) {
                  return Container(
                    height: MediaQuery.of(context).size.height / 2.2,
                    child: constant.paymentComponent(
                        true,
                        context,
                        widget.transactionAmount,
                        _formKey,
                        _autoValidate,
                        phoneController,
                        widget.X_API_KEY,
                        widget.transaction_id,
                        widget.transactionCallBackUrl,
                        widget.provider_short_tag,
                        widget.merchandPassword,
                        widget.merchandUserName,
                        widget.actionAfterProccess, () {
                      setState(() {
                        _autoValidate = true;
                      });
                    }, widget.sandbox),
                  );
                } else if (snapshot.connectionState == ConnectionState.none) {
                  return Text("Turn on Your internet Connexion");
                } else if (snapshot.data == true) {
                  return Container(
                    height: MediaQuery.of(context).size.height / 1.5,
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.center,
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [
                        SizedBox(
                          height: 20,
                        ),
                        CupertinoActivityIndicator(),
                        SizedBox(
                          height: MediaQuery.of(context).size.height * 0.1,
                        ),
                        Text(
                          "Your payment is in progress , please wait ...",
                          style: TextStyle(color: Colors.black),
                          textAlign: TextAlign.center,
                        )
                      ],
                    ),
                  );
                } else {
                  return Container(
                    height: MediaQuery.of(context).size.height / 1.6,
                    child: constant.paymentComponent(
                        false,
                        context,
                        widget.transactionAmount,
                        _formKey,
                        _autoValidate,
                        phoneController,
                        widget.X_API_KEY,
                        widget.transaction_id,
                        widget.transactionCallBackUrl,
                        widget.provider_short_tag,
                        widget.merchandPassword,
                        widget.merchandUserName,
                        widget.actionAfterProccess, () {
                      setState(() {
                        _autoValidate = true;
                      });
                    }, widget.sandbox),
                  );
                }
              })),
    );
  }

  ////////////////////utility class
  progressHub(msg, context) {
    NativeProgressHud.showWaitingWithText(msg);
  }

  closeDialog(context) {
    NativeProgressHud.hideWaiting();
  }

  NumberFormat getOcy() {
    final oCcy = new NumberFormat("#,##0", "en_us");
    return oCcy;
  }
}

//////////////
class PayDialogWithExpressUnion extends StatefulWidget {
  final String X_API_KEY;
  final String transactionAmount;
  final Function actionAfterProccess;
  final String provider_short_tag;
  final String phoneNumber;
  final String transactionCallBackUrl;
  final String merchandUserName;
  final String merchandPassword;
  final String sandbox;
  final String transaction_id;
  final String productName;

  const PayDialogWithExpressUnion({
    this.X_API_KEY,
    this.productName,
    this.transactionAmount,
    this.actionAfterProccess,
    this.provider_short_tag,
    this.transactionCallBackUrl,
    this.phoneNumber,
    this.merchandUserName,
    this.merchandPassword,
    this.sandbox,
    this.transaction_id,
  });

  @override
  _PayDialogWithExpressUnionState createState() =>
      _PayDialogWithExpressUnionState();
}

class _PayDialogWithExpressUnionState extends State<PayDialogWithExpressUnion>
    with AnimationMixin {
  final phoneController = TextEditingController();
  final userNameController = TextEditingController();

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return SingleChildScrollView(
      child: Container(
        height: MediaQuery.of(context).size.height / 1.5,
        child: Padding(
          padding: EdgeInsets.only(left: 30, right: 30),
          child: Column(
            children: <Widget>[
              SizedBox(
                height: MediaQuery.of(context).size.height * 0.05,
              ),
              Text(
                "${getOcy().format((int.parse(widget.transactionAmount)))} XAF",
                style: TextStyle(
                    color: constant.payUnitGreenColor,
                    fontSize: MediaQuery.of(context).size.width * 0.080,
                    fontWeight: FontWeight.bold),
              ),
              SizedBox(
                height: MediaQuery.of(context).size.height * 0.02,
              ),
              Text(
                "You are going to paid ${getOcy().format((int.parse(widget.transactionAmount)))} XAF with Express Union .",
                style: TextStyle(
                    color: Colors.black,
                    fontSize: 14,
                    fontWeight: FontWeight.bold),
              ),
              SizedBox(
                height: MediaQuery.of(context).size.height * 0.04,
              ),
              TextField(
                keyboardType: TextInputType.text,
                controller: userNameController,
                decoration: InputDecoration(
                  contentPadding: EdgeInsets.all(
                      MediaQuery.of(context).size.height * 0.020),
                  hintText: "Enter your User name",
                  hintStyle: TextStyle(
                    color: Colors.grey,
                    fontWeight: FontWeight.bold,
                    fontSize: MediaQuery.of(context).size.width * 0.032,
                  ),
                  border: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(10.0),
                  ),
                  prefixIcon: Icon(
                    FontAwesomeIcons.userCircle,
                    color: constant.payUnitGreenColor,
                    size: 20,
                  ),
                ),
              ),
              SizedBox(
                height: MediaQuery.of(context).size.height * 0.03,
              ),
              TextField(
                keyboardType: TextInputType.number,
                controller: phoneController,
                decoration: InputDecoration(
                  contentPadding: EdgeInsets.all(
                      MediaQuery.of(context).size.height * 0.020),
                  hintText: "Enter your phone number",
                  hintStyle: TextStyle(
                    color: Colors.grey,
                    fontWeight: FontWeight.bold,
                    fontSize: MediaQuery.of(context).size.width * 0.032,
                  ),
                  border: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(10.0),
                  ),
                  prefixIcon: Icon(
                    FontAwesomeIcons.phoneAlt,
                    color: constant.payUnitGreenColor,
                    size: 20,
                  ),
                ),
              ),
              SizedBox(
                height: MediaQuery.of(context).size.height * 0.05,
              ),
              AnimButton(
                color:constant.payUnitGreenColor,
                pressEvent: () {
                  String phoneNumber = phoneController.text;
                  String userName = userNameController.text;
                  if (phoneNumber.isEmpty || userName.isEmpty) {
                    print('Phone number is empty');
                  } else {
                    AwesomeDialog(
                        dismissOnBackKeyPress: false,
                        dismissOnTouchOutside: false,
                        context: context,
                        dialogType: DialogType.INFO,
                        animType: AnimType.BOTTOMSLIDE,
                        title: "Confirm",
                        desc: 'Do you want to confirm this payment',
                        btnOkColor: Colors.green,
                        btnOkText: "Confirm",
                        btnOkIcon: Icons.check,
                        btnOkOnPress: () {
                          ApiService api = new ApiService();
                          api.makePayment(
                              X_API_KEY: widget.X_API_KEY,
                              transactionAmount: widget.transactionAmount,
                              transactionCallBackUrl:
                              widget.transactionCallBackUrl,
                              phoneNumber: phoneController.text,
                              provider_short_tag: widget.provider_short_tag,
                              userName: userName,
                              merchandPassword: widget.merchandPassword,
                              merchandUserName: widget.merchandUserName,
                              mode: widget.sandbox,
                              transaction_id: widget.transaction_id,
                              context: context,
                              actionAfterProccess: widget.actionAfterProccess);
                        },
                        btnCancelOnPress: () {},
                        btnCancelColor: Colors.red,
                        btnCancelIcon: Icons.cancel,
                        btnCancelText: 'Cancel')
                      ..show();
                  }
                },
                text: 'CONTINUE',
              )
            ],
          ),
        ),
      ),
    );
  }
}

progressHub(msg, context) {
  NativeProgressHud.showWaitingWithText(msg);
}

closeDialog(context) {
  NativeProgressHud.hideWaiting();
}

NumberFormat getOcy() {
  final oCcy = new NumberFormat("#,##0", "en_us");
  return oCcy;
}

class AnimButton extends StatefulWidget {
  final Function pressEvent;
  final String text;
  final IconData icon;
  final double width;
  final bool isFixedHeight;
  final Color color;

  const AnimButton(
      {@required this.pressEvent,
        this.text,
        this.icon,
        this.color,
        this.isFixedHeight = true,
        this.width = double.infinity});

  @override
  _AnimatedButtonState createState() => _AnimatedButtonState();
}

class _AnimatedButtonState extends State<AnimButton> with AnimationMixin {
  Animation<double> _scale;

  @override
  void initState() {
    super.initState();
    final curveAnimation = CurvedAnimation(
        parent: controller, curve: Curves.easeIn, reverseCurve: Curves.easeIn);
    _scale = Tween<double>(begin: 1, end: 0.9).animate(curveAnimation);
  }

  void _onTapDown(TapDownDetails details) {
    controller.play(duration: Duration(milliseconds: 150));
  }

  void _onTapUp(TapUpDetails details) {
    if (controller.isAnimating) {
      controller.addStatusListener((status) {
        if (status == AnimationStatus.completed)
          controller.playReverse(duration: Duration(milliseconds: 100));
      });
    } else
      controller.playReverse(duration: Duration(milliseconds: 100));
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        widget.pressEvent();
        //  _controller.forward();
      },
      onTapDown: _onTapDown,
      onTapUp: _onTapUp,
      onTapCancel: () {
        controller.playReverse(
          duration: Duration(milliseconds: 100),
        );
      },
      child: Transform.scale(
        scale: _scale.value,
        child: _animatedButtonUI,
      ),
    );
  }

  Widget get _animatedButtonUI => Container(
    height: widget.isFixedHeight ? 50.0 : null,
    width: widget.width,
    padding: const EdgeInsets.all(8.0),
    decoration: BoxDecoration(
        borderRadius: BorderRadius.all(Radius.circular(10)),
        color: widget.color ?? Theme.of(context).primaryColor),
    child: Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: <Widget>[
        widget.icon != null
            ? Padding(
          padding: const EdgeInsets.only(left: 4.0),
          child: Icon(
            widget.icon,
            color: Colors.white,
          ),
        )
            : SizedBox(),
        SizedBox(
          width: 5,
        ),
        Flexible(
          fit: FlexFit.loose,
          child: Text(
            '${widget.text}',
            // maxLines: 1,
            textAlign: TextAlign.center,
            style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
                fontSize: 14),
          ),
        ),
      ],
    ),
  );
}
